@@ -7,73 +7,63 @@ namespace mold::elf {
 
 using E = ARM64;
 
-static void write_adr(u8 *buf, u64 val) {
-  u32 hi = (val & 0x1ffffc) << 3;
-  u32 lo = (val & 3) << 29;
-  *(u32 *)buf = (*(u32 *)buf & 0x9f00001f) | hi | lo;
+static void write_adrp(u8 *buf, u64 val) {
+  u32 hi = bits(val, 32, 14);
+  u32 lo = bits(val, 13, 12);
+  u32 op = *(ul32 *)buf & 0b1001'1111'0000'0000'0000'0000'0001'1111;
+  *(ul32 *)buf = (lo << 29) | (hi << 5) | op;
 }
 
-// Returns [hi:lo] bits of val.
-static u64 bits(u64 val, u64 hi, u64 lo) {
-  return (val >> lo) & (((u64)1 << (hi - lo + 1)) - 1);
+static void write_adr(u8 *buf, u64 val) {
+  u32 hi = bits(val, 20, 2);
+  u32 lo = bits(val, 1, 0);
+  u32 op = *(ul32 *)buf & 0b1001'1111'0000'0000'0000'0000'0001'1111;
+  *(ul32 *)buf = (lo << 29) | (hi << 5) | op;
 }
 
 static u64 page(u64 val) {
   return val & ~(u64)0xfff;
 }
 
-template <>
-void GotPltSection<E>::copy_buf(Context<E> &ctx) {
-  u64 *buf = (u64 *)(ctx.buf + this->shdr.sh_offset);
-
-  // The first slot of .got.plt points to _DYNAMIC.
-  buf[0] = ctx.dynamic ? ctx.dynamic->shdr.sh_addr : 0;
-  buf[1] = 0;
-  buf[2] = 0;
-
-  for (Symbol<E> *sym : ctx.plt->symbols)
-    buf[sym->get_gotplt_idx(ctx)] = ctx.plt->shdr.sh_addr;
-}
-
 static void write_plt_header(Context<E> &ctx, u8 *buf) {
   // Write PLT header
-  static const u8 plt0[] = {
-    0xf0, 0x7b, 0xbf, 0xa9, // stp    x16, x30, [sp,#-16]!
-    0x10, 0x00, 0x00, 0x90, // adrp   x16, .got.plt[2]
-    0x11, 0x02, 0x40, 0xf9, // ldr    x17, [x16, .got.plt[2]]
-    0x10, 0x02, 0x00, 0x91, // add    x16, x16, .got.plt[2]
-    0x20, 0x02, 0x1f, 0xd6, // br     x17
-    0x1f, 0x20, 0x03, 0xd5, // nop
-    0x1f, 0x20, 0x03, 0xd5, // nop
-    0x1f, 0x20, 0x03, 0xd5, // nop
+  static const u32 plt0[] = {
+    0xa9bf7bf0, // stp  x16, x30, [sp,#-16]!
+    0x90000010, // adrp x16, .got.plt[2]
+    0xf9400211, // ldr  x17, [x16, .got.plt[2]]
+    0x91000210, // add  x16, x16, .got.plt[2]
+    0xd61f0220, // br   x17
+    0xd503201f, // nop
+    0xd503201f, // nop
+    0xd503201f, // nop
   };
 
   u64 gotplt = ctx.gotplt->shdr.sh_addr + 16;
   u64 plt = ctx.plt->shdr.sh_addr;
 
   memcpy(buf, plt0, sizeof(plt0));
-  write_adr(buf + 4, bits(page(gotplt) - page(plt + 4), 32, 12));
-  *(u32 *)(buf + 8) |= bits(gotplt, 11, 3) << 10;
-  *(u32 *)(buf + 12) |= ((gotplt) & 0xfff) << 10;
+  write_adrp(buf + 4, page(gotplt) - page(plt + 4));
+  *(ul32 *)(buf + 8) |= bits(gotplt, 11, 3) << 10;
+  *(ul32 *)(buf + 12) |= (gotplt & 0xfff) << 10;
 }
 
 static void write_plt_entry(Context<E> &ctx, u8 *buf, Symbol<E> &sym) {
   u8 *ent = buf + ctx.plt_hdr_size + sym.get_plt_idx(ctx) * ctx.plt_size;
 
-  static const u8 data[] = {
-    0x10, 0x00, 0x00, 0x90, // adrp x16, .got.plt[n]
-    0x11, 0x02, 0x40, 0xf9, // ldr  x17, [x16, .got.plt[n]]
-    0x10, 0x02, 0x00, 0x91, // add  x16, x16, .got.plt[n]
-    0x20, 0x02, 0x1f, 0xd6, // br   x17
+  static const u32 data[] = {
+    0x90000010, // adrp x16, .got.plt[n]
+    0xf9400211, // ldr  x17, [x16, .got.plt[n]]
+    0x91000210, // add  x16, x16, .got.plt[n]
+    0xd61f0220, // br   x17
   };
 
   u64 gotplt = sym.get_gotplt_addr(ctx);
   u64 plt = sym.get_plt_addr(ctx);
 
   memcpy(ent, data, sizeof(data));
-  write_adr(ent, bits(page(gotplt) - page(plt), 32, 12));
-  *(u32 *)(ent + 4) |= bits(gotplt, 11, 3) << 10;
-  *(u32 *)(ent + 8) |= (gotplt & 0xfff) << 10;
+  write_adrp(ent, page(gotplt) - page(plt));
+  *(ul32 *)(ent + 4) |= bits(gotplt, 11, 3) << 10;
+  *(ul32 *)(ent + 8) |= (gotplt & 0xfff) << 10;
 }
 
 template <>
@@ -91,36 +81,36 @@ void PltGotSection<E>::copy_buf(Context<E> &ctx) {
   for (Symbol<E> *sym : symbols) {
     u8 *ent = buf + sym->get_pltgot_idx(ctx) * ARM64::pltgot_size;
 
-    static const u8 data[] = {
-      0x10, 0x00, 0x00, 0x90, // adrp x16, GOT[n]
-      0x11, 0x02, 0x40, 0xf9, // ldr  x17, [x16, GOT[n]]
-      0x20, 0x02, 0x1f, 0xd6, // br   x17
-      0x1f, 0x20, 0x03, 0xd5, // nop
+    static const u32 data[] = {
+      0x90000010, // adrp x16, GOT[n]
+      0xf9400211, // ldr  x17, [x16, GOT[n]]
+      0xd61f0220, // br   x17
+      0xd503201f, // nop
     };
 
     u64 got = sym->get_got_addr(ctx);
     u64 plt = sym->get_plt_addr(ctx);
 
     memcpy(ent, data, sizeof(data));
-    write_adr(ent, bits(page(got) - page(plt), 32, 12));
-    *(u32 *)(ent + 4) |= bits(got, 11, 3) << 10;
+    write_adrp(ent, page(got) - page(plt));
+    *(ul32 *)(ent + 4) |= bits(got, 11, 3) << 10;
   }
 }
 
 template <>
-void EhFrameSection<E>::apply_reloc(Context<E> &ctx, ElfRel<E> &rel,
-                                    u64 loc, u64 val) {
-  u8 *base = ctx.buf + this->shdr.sh_offset;
+void EhFrameSection<E>::apply_reloc(Context<E> &ctx, const ElfRel<E> &rel,
+                                    u64 offset, u64 val) {
+  u8 *loc = ctx.buf + this->shdr.sh_offset + offset;
 
   switch (rel.r_type) {
   case R_AARCH64_ABS64:
-    *(u64 *)(base + loc) = val;
+    *(ul64 *)loc = val;
     return;
   case R_AARCH64_PREL32:
-    *(u32 *)(base + loc) = val - this->shdr.sh_addr - loc;
+    *(ul32 *)loc = val - this->shdr.sh_addr - offset;
     return;
   case R_AARCH64_PREL64:
-    *(u64 *)(base + loc) = val - this->shdr.sh_addr - loc;
+    *(ul64 *)loc = val - this->shdr.sh_addr - offset;
     return;
   }
   Fatal(ctx) << "unsupported relocation in .eh_frame: " << rel;
@@ -129,7 +119,7 @@ void EhFrameSection<E>::apply_reloc(Context<E> &ctx, ElfRel<E> &rel,
 template <>
 void InputSection<E>::apply_reloc_alloc(Context<E> &ctx, u8 *base) {
   ElfRel<E> *dynrel = nullptr;
-  std::span<ElfRel<E>> rels = get_rels(ctx);
+  std::span<const ElfRel<E>> rels = get_rels(ctx);
 
   i64 frag_idx = 0;
 
@@ -157,68 +147,67 @@ void InputSection<E>::apply_reloc_alloc(Context<E> &ctx, u8 *base) {
     };
 
 #define S   (frag_ref ? frag_ref->frag->get_addr(ctx) : sym.get_addr(ctx))
-#define A   (frag_ref ? frag_ref->addend : rel.r_addend)
+#define A   (frag_ref ? (u64)frag_ref->addend : (u64)rel.r_addend)
 #define P   (output_section->shdr.sh_addr + offset + rel.r_offset)
 #define G   (sym.get_got_addr(ctx) - ctx.got->shdr.sh_addr)
 #define GOT ctx.got->shdr.sh_addr
 
-    if (needs_dynrel[i]) {
-      *dynrel++ = {P, R_AARCH64_ABS64, (u32)sym.get_dynsym_idx(ctx), A};
-      *(u64 *)loc = A;
-      continue;
-    }
-
-    if (needs_baserel[i]) {
-      if (!is_relr_reloc(ctx, rel))
-        *dynrel++ = {P, R_AARCH64_RELATIVE, 0, (i64)(S + A)};
-      *(u64 *)loc = S + A;
-      continue;
-    }
-
     switch (rel.r_type) {
     case R_AARCH64_ABS64:
-      *(u64 *)loc = S + A;
+      if (sym.is_absolute() || !ctx.arg.pic) {
+        *(ul64 *)loc = S + A;
+      } else if (sym.is_imported) {
+        *dynrel++ = {P, R_AARCH64_ABS64, (u32)sym.get_dynsym_idx(ctx), A};
+        *(ul64 *)loc = A;
+      } else {
+        if (!is_relr_reloc(ctx, rel))
+          *dynrel++ = {P, R_AARCH64_RELATIVE, 0, (i64)(S + A)};
+        *(ul64 *)loc = S + A;
+      }
       continue;
     case R_AARCH64_LDST8_ABS_LO12_NC:
-      *(u32 *)loc |= bits(S + A, 11, 0) << 10;
+      *(ul32 *)loc |= bits(S + A, 11, 0) << 10;
       continue;
     case R_AARCH64_LDST16_ABS_LO12_NC:
-      *(u32 *)loc |= bits(S + A, 11, 1) << 10;
+      *(ul32 *)loc |= bits(S + A, 11, 1) << 10;
       continue;
     case R_AARCH64_LDST32_ABS_LO12_NC:
-      *(u32 *)loc |= bits(S + A, 11, 2) << 10;
+      *(ul32 *)loc |= bits(S + A, 11, 2) << 10;
       continue;
     case R_AARCH64_LDST64_ABS_LO12_NC:
-      *(u32 *)loc |= bits(S + A, 11, 3) << 10;
+      *(ul32 *)loc |= bits(S + A, 11, 3) << 10;
       continue;
     case R_AARCH64_LDST128_ABS_LO12_NC:
-      *(u32 *)loc |= bits(S + A, 11, 4) << 10;
+      *(ul32 *)loc |= bits(S + A, 11, 4) << 10;
       continue;
     case R_AARCH64_ADD_ABS_LO12_NC:
-      *(u32 *)loc |= bits(S + A, 11, 0) << 10;
+      *(ul32 *)loc |= bits(S + A, 11, 0) << 10;
       continue;
+    case R_AARCH64_MOVW_UABS_G0:
     case R_AARCH64_MOVW_UABS_G0_NC:
-      *(u32 *)loc |= bits(S + A, 15, 0) << 5;
+      *(ul32 *)loc |= bits(S + A, 15, 0) << 5;
       continue;
+    case R_AARCH64_MOVW_UABS_G1:
     case R_AARCH64_MOVW_UABS_G1_NC:
-      *(u32 *)loc |= bits(S + A, 31, 16) << 5;
+      *(ul32 *)loc |= bits(S + A, 31, 16) << 5;
       continue;
+    case R_AARCH64_MOVW_UABS_G2:
     case R_AARCH64_MOVW_UABS_G2_NC:
-      *(u32 *)loc |= bits(S + A, 47, 32) << 5;
+      *(ul32 *)loc |= bits(S + A, 47, 32) << 5;
       continue;
     case R_AARCH64_MOVW_UABS_G3:
-      *(u32 *)loc |= bits(S + A, 63, 48) << 5;
+      *(ul32 *)loc |= bits(S + A, 63, 48) << 5;
       continue;
     case R_AARCH64_ADR_GOT_PAGE: {
       i64 val = page(G + GOT + A) - page(P);
       overflow_check(val, -((i64)1 << 32), (i64)1 << 32);
-      write_adr(loc, bits(val, 32, 12));
+      write_adrp(loc, val);
       continue;
     }
     case R_AARCH64_ADR_PREL_PG_HI21: {
       i64 val = page(S + A) - page(P);
       overflow_check(val, -((i64)1 << 32), (i64)1 << 32);
-      write_adr(loc, bits(val, 32, 12));
+      write_adrp(loc, val);
       continue;
     }
     case R_AARCH64_ADR_PREL_LO21: {
@@ -232,7 +221,7 @@ void InputSection<E>::apply_reloc_alloc(Context<E> &ctx, u8 *base) {
       if (sym.esym().is_undef_weak()) {
         // On ARM, calling an weak undefined symbol jumps to the
         // next instruction.
-        *(u32 *)loc |= 1;
+        *(ul32 *)loc |= 1;
         continue;
       }
 
@@ -241,81 +230,83 @@ void InputSection<E>::apply_reloc_alloc(Context<E> &ctx, u8 *base) {
       i64 val = S + A - P;
 
       if (val < lo || hi <= val) {
-        RangeExtensionRef ref = range_extn[i];
+        RangeExtensionRef ref = extra.range_extn[i];
         val = output_section->thunks[ref.thunk_idx]->get_addr(ref.sym_idx) + A - P;
         assert(lo <= val && val < hi);
       }
 
-      *(u32 *)loc |= (val >> 2) & 0x3ffffff;
+      *(ul32 *)loc |= (val >> 2) & 0x3ffffff;
       continue;
     }
-    case R_AARCH64_CONDBR19: {
+    case R_AARCH64_CONDBR19:
+    case R_AARCH64_LD_PREL_LO19: {
       i64 val = S + A - P;
       overflow_check(val, -((i64)1 << 20), (i64)1 << 20);
-      *(u32 *)loc |= bits(val, 20, 2) << 5;
+      *(ul32 *)loc |= bits(val, 20, 2) << 5;
       continue;
     }
     case R_AARCH64_PREL16: {
       i64 val = S + A - P;
       overflow_check(val, -((i64)1 << 15), (i64)1 << 15);
-      *(u16 *)loc = val;
+      *(ul16 *)loc = val;
       continue;
     }
     case R_AARCH64_PREL32: {
       i64 val = S + A - P;
       overflow_check(val, -((i64)1 << 31), (i64)1 << 32);
-      *(u32 *)loc = val;
+      *(ul32 *)loc = val;
       continue;
     }
     case R_AARCH64_PREL64:
-      *(u64 *)loc = S + A - P;
+      *(ul64 *)loc = S + A - P;
       continue;
     case R_AARCH64_LD64_GOT_LO12_NC:
-      *(u32 *)loc |= bits(G + GOT + A, 11, 3) << 10;
+      *(ul32 *)loc |= bits(G + GOT + A, 11, 3) << 10;
       continue;
     case R_AARCH64_LD64_GOTPAGE_LO15: {
       i64 val = G + GOT + A - page(GOT);
       overflow_check(val, 0, 1 << 15);
-      *(u32 *)loc |= bits(val, 14, 3) << 10;
+      *(ul32 *)loc |= bits(val, 14, 3) << 10;
       continue;
     }
     case R_AARCH64_TLSIE_ADR_GOTTPREL_PAGE21: {
       i64 val = page(sym.get_gottp_addr(ctx) + A) - page(P);
       overflow_check(val, -((i64)1 << 32), (i64)1 << 32);
-      write_adr(loc, bits(val, 32, 12));
+      write_adrp(loc, val);
       continue;
     }
     case R_AARCH64_TLSIE_LD64_GOTTPREL_LO12_NC:
-      *(u32 *)loc |= bits(sym.get_gottp_addr(ctx) + A, 11, 3) << 10;
+      *(ul32 *)loc |= bits(sym.get_gottp_addr(ctx) + A, 11, 3) << 10;
       continue;
     case R_AARCH64_TLSLE_ADD_TPREL_HI12: {
       i64 val = S + A - ctx.tls_begin + 16;
       overflow_check(val, 0, (i64)1 << 24);
-      *(u32 *)loc |= bits(val, 23, 12) << 10;
+      *(ul32 *)loc |= bits(val, 23, 12) << 10;
       continue;
     }
+    case R_AARCH64_TLSLE_ADD_TPREL_LO12:
     case R_AARCH64_TLSLE_ADD_TPREL_LO12_NC:
-      *(u32 *)loc |= bits(S + A - ctx.tls_begin + 16, 11, 0) << 10;
+      *(ul32 *)loc |= bits(S + A - ctx.tls_begin + 16, 11, 0) << 10;
       continue;
     case R_AARCH64_TLSGD_ADR_PAGE21: {
       i64 val = page(sym.get_tlsgd_addr(ctx) + A) - page(P);
       overflow_check(val, -((i64)1 << 32), (i64)1 << 32);
-      write_adr(loc, bits(val, 32, 12));
+      write_adrp(loc, val);
       continue;
     }
     case R_AARCH64_TLSGD_ADD_LO12_NC:
-      *(u32 *)loc |= bits(sym.get_tlsgd_addr(ctx) + A, 11, 0) << 10;
+      *(ul32 *)loc |= bits(sym.get_tlsgd_addr(ctx) + A, 11, 0) << 10;
       continue;
     case R_AARCH64_TLSDESC_ADR_PAGE21: {
       if (ctx.relax_tlsdesc && !sym.is_imported) {
         // adrp x0, 0 -> movz x0, #tls_ofset_hi, lsl #16
         i64 val = (S + A - ctx.tls_begin + 16);
         overflow_check(val, -((i64)1 << 32), (i64)1 << 32);
-        *(u32 *)loc = 0xd2a00000 | (bits(val, 32, 16) << 5);
+        *(ul32 *)loc = 0xd2a00000 | (bits(val, 32, 16) << 5);
       } else {
         i64 val = page(sym.get_tlsdesc_addr(ctx) + A) - page(P);
         overflow_check(val, -((i64)1 << 32), (i64)1 << 32);
-        write_adr(loc, bits(val, 32, 12));
+        write_adrp(loc, val);
       }
       continue;
     }
@@ -323,23 +314,23 @@ void InputSection<E>::apply_reloc_alloc(Context<E> &ctx, u8 *base) {
       if (ctx.relax_tlsdesc && !sym.is_imported) {
         // ldr x2, [x0] -> movk x0, #tls_ofset_lo
         u32 offset_lo = (S + A - ctx.tls_begin + 16) & 0xffff;
-        *(u32 *)loc = 0xf2800000 | (offset_lo << 5);
+        *(ul32 *)loc = 0xf2800000 | (offset_lo << 5);
       } else {
-        *(u32 *)loc |= bits(sym.get_tlsdesc_addr(ctx) + A, 11, 3) << 10;
+        *(ul32 *)loc |= bits(sym.get_tlsdesc_addr(ctx) + A, 11, 3) << 10;
       }
       continue;
     case R_AARCH64_TLSDESC_ADD_LO12:
       if (ctx.relax_tlsdesc && !sym.is_imported) {
         // add x0, x0, #0 -> nop
-        *(u32 *)loc = 0xd503201f;
+        *(ul32 *)loc = 0xd503201f;
       } else {
-        *(u32 *)loc |= bits(sym.get_tlsdesc_addr(ctx) + A, 11, 0) << 10;
+        *(ul32 *)loc |= bits(sym.get_tlsdesc_addr(ctx) + A, 11, 0) << 10;
       }
       continue;
     case R_AARCH64_TLSDESC_CALL:
       if (ctx.relax_tlsdesc && !sym.is_imported) {
         // blr x2 -> nop
-        *(u32 *)loc = 0xd503201f;
+        *(ul32 *)loc = 0xd503201f;
       }
       continue;
     default:
@@ -356,8 +347,7 @@ void InputSection<E>::apply_reloc_alloc(Context<E> &ctx, u8 *base) {
 
 template <>
 void InputSection<E>::apply_reloc_nonalloc(Context<E> &ctx, u8 *base) {
-  std::span<ElfRel<E>> rels = get_rels(ctx);
-  i64 frag_idx = 0;
+  std::span<const ElfRel<E>> rels = get_rels(ctx);
 
   for (i64 i = 0; i < rels.size(); i++) {
     const ElfRel<E> &rel = rels[i];
@@ -368,7 +358,7 @@ void InputSection<E>::apply_reloc_nonalloc(Context<E> &ctx, u8 *base) {
     u8 *loc = base + rel.r_offset;
 
     if (!sym.file) {
-      report_undef(ctx, sym);
+      record_undef_error(ctx, rel);
       continue;
     }
 
@@ -377,14 +367,20 @@ void InputSection<E>::apply_reloc_nonalloc(Context<E> &ctx, u8 *base) {
     std::tie(frag, addend) = get_fragment(ctx, rel);
 
 #define S (frag ? frag->get_addr(ctx) : sym.get_addr(ctx))
-#define A (frag ? addend : rel.r_addend)
+#define A (frag ? (u64)addend : (u64)rel.r_addend)
 
     switch (rel.r_type) {
     case R_AARCH64_ABS64:
-      *(u64 *)loc = S + A;
+      if (!frag) {
+        if (std::optional<u64> val = get_tombstone(sym)) {
+          *(ul64 *)loc = *val;
+          break;
+        }
+      }
+      *(ul64 *)loc = S + A;
       continue;
     case R_AARCH64_ABS32:
-      *(u32 *)loc = S + A;
+      *(ul32 *)loc = S + A;
       continue;
     default:
       Fatal(ctx) << *this << ": invalid relocation for non-allocated sections: "
@@ -399,10 +395,10 @@ void InputSection<E>::apply_reloc_nonalloc(Context<E> &ctx, u8 *base) {
 
 template <>
 void InputSection<E>::scan_relocations(Context<E> &ctx) {
-  assert(shdr.sh_flags & SHF_ALLOC);
+  assert(shdr().sh_flags & SHF_ALLOC);
 
   this->reldyn_offset = file.num_dynrel * sizeof(ElfRel<E>);
-  std::span<ElfRel<E>> rels = get_rels(ctx);
+  std::span<const ElfRel<E>> rels = get_rels(ctx);
 
   // Scan relocations
   for (i64 i = 0; i < rels.size(); i++) {
@@ -413,7 +409,7 @@ void InputSection<E>::scan_relocations(Context<E> &ctx) {
     Symbol<E> &sym = *file.symbols[rel.r_sym];
 
     if (!sym.file) {
-      report_undef(ctx, sym);
+      record_undef_error(ctx, rel);
       continue;
     }
 
@@ -428,7 +424,7 @@ void InputSection<E>::scan_relocations(Context<E> &ctx) {
         // Absolute  Local    Imported data  Imported code
         {  NONE,     BASEREL, DYNREL,        DYNREL },     // DSO
         {  NONE,     BASEREL, DYNREL,        DYNREL },     // PIE
-        {  NONE,     NONE,    COPYREL,       PLT    },     // PDE
+        {  NONE,     NONE,    COPYREL,       CPLT   },     // PDE
       };
       dispatch(ctx, table, i, rel, sym);
       break;
@@ -450,9 +446,9 @@ void InputSection<E>::scan_relocations(Context<E> &ctx) {
     case R_AARCH64_ADR_PREL_PG_HI21: {
       Action table[][4] = {
         // Absolute  Local    Imported data  Imported code
-        {  NONE,     NONE,    ERROR,         ERROR },      // DSO
-        {  NONE,     NONE,    COPYREL,       PLT   },      // PIE
-        {  NONE,     NONE,    COPYREL,       PLT   },      // PDE
+        {  ERROR,    NONE,    ERROR,         ERROR },      // DSO
+        {  ERROR,    NONE,    COPYREL,       PLT   },      // PIE
+        {  NONE,     NONE,    COPYREL,       CPLT  },      // PDE
       };
       dispatch(ctx, table, i, rel, sym);
       break;
@@ -469,19 +465,24 @@ void InputSection<E>::scan_relocations(Context<E> &ctx) {
     case R_AARCH64_ADD_ABS_LO12_NC:
     case R_AARCH64_ADR_PREL_LO21:
     case R_AARCH64_CONDBR19:
+    case R_AARCH64_LD_PREL_LO19:
     case R_AARCH64_LDST16_ABS_LO12_NC:
     case R_AARCH64_LDST32_ABS_LO12_NC:
     case R_AARCH64_LDST64_ABS_LO12_NC:
     case R_AARCH64_LDST128_ABS_LO12_NC:
     case R_AARCH64_LDST8_ABS_LO12_NC:
+    case R_AARCH64_MOVW_UABS_G0:
     case R_AARCH64_MOVW_UABS_G0_NC:
+    case R_AARCH64_MOVW_UABS_G1:
     case R_AARCH64_MOVW_UABS_G1_NC:
+    case R_AARCH64_MOVW_UABS_G2:
     case R_AARCH64_MOVW_UABS_G2_NC:
     case R_AARCH64_MOVW_UABS_G3:
     case R_AARCH64_PREL16:
     case R_AARCH64_PREL32:
     case R_AARCH64_PREL64:
     case R_AARCH64_TLSLE_ADD_TPREL_HI12:
+    case R_AARCH64_TLSLE_ADD_TPREL_LO12:
     case R_AARCH64_TLSLE_ADD_TPREL_LO12_NC:
     case R_AARCH64_TLSGD_ADD_LO12_NC:
     case R_AARCH64_TLSDESC_CALL:
@@ -494,19 +495,25 @@ void InputSection<E>::scan_relocations(Context<E> &ctx) {
 
 static void reset_thunk(RangeExtensionThunk<E> &thunk) {
   for (Symbol<E> *sym : thunk.symbols) {
-    sym->thunk_idx = -1;
-    sym->thunk_sym_idx = -1;
-    sym->flags &= (u8)~NEEDS_THUNK;
+    sym->extra.thunk_idx = -1;
+    sym->extra.thunk_sym_idx = -1;
+    sym->flags &= (u8)~NEEDS_RANGE_EXTN_THUNK;
   }
 }
 
 static bool is_reachable(Context<E> &ctx, Symbol<E> &sym,
                          InputSection<E> &isec, const ElfRel<E> &rel) {
-  // We always create a thunk for an absolute symbol conservatively
-  // because `shrink_sections` may increase a distance between a
-  // branch instruction and an absolute symbol. Branching to an
-  // absolute location is extremely rare in real code, though.
-  if (sym.is_absolute())
+  // We pessimistically assume that PLT entries are unreacahble.
+  if (sym.has_plt(ctx))
+    return false;
+
+  // We create thunks with a pessimistic assumption that all
+  // out-of-section relocations would be out-of-range.
+  InputSection<E> *isec2 = sym.get_input_section();
+  if (!isec2 || isec.output_section != isec2->output_section)
+    return false;
+
+  if (isec2->offset == -1)
     return false;
 
   // Compute a distance between the relocated place and the symbol
@@ -519,20 +526,28 @@ static bool is_reachable(Context<E> &ctx, Symbol<E> &sym,
 }
 
 // We create a thunk no further than 100 MiB from any section.
-constexpr i64 MAX_DISTANCE = 100 * 1024 * 1024;
+static constexpr i64 MAX_DISTANCE = 100 * 1024 * 1024;
 
 // We create a thunk for each 10 MiB input sections.
 static constexpr i64 GROUP_SIZE = 10 * 1024 * 1024;
 
-static void create_thunks(Context<E> &ctx, OutputSection<E> &osec) {
+// ARM64's call/jump instructions take 27 bits displacement, so they
+// can refer only up to ±128 MiB. If a branch target is further than
+// that, we need to let it branch to a linker-synthesized code
+// sequence that construct a full 32 bit address in a register and
+// jump there. That linker-synthesized code is called "thunk".
+void create_range_extension_thunks(Context<E> &ctx, OutputSection<E> &osec) {
   std::span<InputSection<E> *> members = osec.members;
+  if (members.empty())
+    return;
+
   members[0]->offset = 0;
 
-  // Initialize input sections with very large dummy offsets so that
-  // sections that have got real offsets are separated from the ones
-  // without in the virtual address space.
+  // Initialize input sections with a dummy offset so that we can
+  // distinguish sections that have got an address with the one who
+  // haven't.
   tbb::parallel_for((i64)1, (i64)members.size(), [&](i64 i) {
-    members[i]->offset = 1 << 31;
+    members[i]->offset = -1;
   });
 
   // We create thunks from the beginning of the section to the end.
@@ -547,9 +562,9 @@ static void create_thunks(Context<E> &ctx, OutputSection<E> &osec) {
   while (b < members.size()) {
     // Move D foward as far as we can jump from B to D.
     while (d < members.size() && offset - members[b]->offset < MAX_DISTANCE) {
-      offset = align_to(offset, members[d]->shdr.sh_addralign);
+      offset = align_to(offset, 1 << members[d]->p2align);
       members[d]->offset = offset;
-      offset += members[d]->shdr.sh_size;
+      offset += members[d]->sh_size;
       d++;
     }
 
@@ -560,7 +575,7 @@ static void create_thunks(Context<E> &ctx, OutputSection<E> &osec) {
 
     // Move A forward so that A is reachable from C.
     if (c > 0) {
-      i64 c_end = members[c - 1]->offset + members[c - 1]->shdr.sh_size;
+      i64 c_end = members[c - 1]->offset + members[c - 1]->sh_size;
       while (a < osec.thunks.size() &&
              osec.thunks[a]->offset < c_end - MAX_DISTANCE)
         reset_thunk(*osec.thunks[a++]);
@@ -574,9 +589,11 @@ static void create_thunks(Context<E> &ctx, OutputSection<E> &osec) {
     thunk.offset = offset;
 
     // Scan relocations between B and C to collect symbols that need thunks.
-    tbb::parallel_for_each(&members[b], &members[c], [&](InputSection<E> *isec) {
-      std::span<ElfRel<E>> rels = isec->get_rels(ctx);
-      isec->range_extn.resize(rels.size());
+    tbb::parallel_for_each(members.begin() + b, members.begin() + c,
+                           [&](InputSection<E> *isec) {
+      std::span<const ElfRel<E>> rels = isec->get_rels(ctx);
+      std::vector<RangeExtensionRef> &range_extn = isec->extra.range_extn;
+      range_extn.resize(rels.size());
 
       for (i64 i = 0; i < rels.size(); i++) {
         const ElfRel<E> &rel = rels[i];
@@ -585,20 +602,25 @@ static void create_thunks(Context<E> &ctx, OutputSection<E> &osec) {
 
         Symbol<E> &sym = *isec->file.symbols[rel.r_sym];
 
+        // Skip if the symbol is undefined. apply_reloc() will report an error.
+        if (!sym.file)
+          continue;
+
         // Skip if the destination is within reach.
         if (is_reachable(ctx, sym, *isec, rel))
           continue;
 
         // If the symbol is already in another thunk, reuse it.
-        if (sym.thunk_idx != -1) {
-          isec->range_extn[i] = {sym.thunk_idx, sym.thunk_sym_idx};
+        if (sym.extra.thunk_idx != -1) {
+          range_extn[i] = {sym.extra.thunk_idx, sym.extra.thunk_sym_idx};
           continue;
         }
 
         // Otherwise, add the symbol to this thunk if it's not added already.
-        isec->range_extn[i] = {thunk.thunk_idx, -1};
+        range_extn[i] = {thunk.thunk_idx, -1};
 
-        if (!(sym.flags.fetch_or(NEEDS_THUNK) & NEEDS_THUNK)) {
+        if (!(sym.flags.fetch_or(NEEDS_RANGE_EXTN_THUNK) &
+              NEEDS_RANGE_EXTN_THUNK)) {
           std::scoped_lock lock(thunk.mu);
           thunk.symbols.push_back(&sym);
         }
@@ -610,24 +632,27 @@ static void create_thunks(Context<E> &ctx, OutputSection<E> &osec) {
     offset += thunk.size();
 
     // Sort symbols added to the thunk to make the output deterministic.
-    sort(thunk.symbols, [](Symbol<E> *a, Symbol<E> *b) { return *a < *b; });
+    sort(thunk.symbols, [](Symbol<E> *a, Symbol<E> *b) {
+      return std::tuple{a->file->priority, a->sym_idx} <
+             std::tuple{b->file->priority, b->sym_idx};
+    });
 
     // Assign offsets within the thunk to the symbols.
     for (i64 i = 0; Symbol<E> *sym : thunk.symbols) {
-      sym->thunk_idx = thunk.thunk_idx;
-      sym->thunk_sym_idx = i++;
+      sym->extra.thunk_idx = thunk.thunk_idx;
+      sym->extra.thunk_sym_idx = i++;
     }
 
     // Scan relocations again to fix symbol offsets in the last thunk.
-    tbb::parallel_for_each(&members[b], &members[c], [&](InputSection<E> *isec) {
-      std::span<ElfRel<E>> rels = isec->get_rels(ctx);
+    tbb::parallel_for_each(members.begin() + b, members.begin() + c,
+                           [&](InputSection<E> *isec) {
+      std::span<const ElfRel<E>> rels = isec->get_rels(ctx);
 
       for (i64 i = 0; i < rels.size(); i++) {
-        RangeExtensionRef &ref = isec->range_extn[i];
-
-        if (ref.thunk_idx == thunk.thunk_idx) {
+        std::vector<RangeExtensionRef> &range_extn = isec->extra.range_extn;
+        if (range_extn[i].thunk_idx == thunk.thunk_idx) {
           Symbol<E> &sym = *isec->file.symbols[rels[i].r_sym];
-          ref.sym_idx = sym.thunk_sym_idx;
+          range_extn[i].sym_idx = sym.extra.thunk_sym_idx;
         }
       }
     });
@@ -642,153 +667,26 @@ static void create_thunks(Context<E> &ctx, OutputSection<E> &osec) {
   osec.shdr.sh_size = offset;
 }
 
-static void mark_thunk_symbols(Context<E> &ctx, OutputSection<E> &osec) {
-  for (std::unique_ptr<RangeExtensionThunk<E>> &thunk : osec.thunks) {
-    i64 sz = thunk->symbols.size();
-    thunk->symbol_map.resize(sz);
-    thunk->used.reset(new std::atomic_bool[sz]{});
-  }
-
-  // Mark referenced thunk symbols
-  tbb::parallel_for_each(osec.members, [&](InputSection<E> *isec) {
-    std::span<ElfRel<E>> rels = isec->get_rels(ctx);
-
-    for (i64 i = 0; i < rels.size(); i++) {
-      RangeExtensionRef &ref = isec->range_extn[i];
-      if (ref.thunk_idx == -1)
-        continue;
-
-      Symbol<E> &sym = *isec->file.symbols[rels[i].r_sym];
-      if (!is_reachable(ctx, sym, *isec, rels[i]))
-        osec.thunks[ref.thunk_idx]->used[ref.sym_idx] = true;
-    }
-  });
-
-  // Remove unreferenced thunk symbols
-  tbb::parallel_for_each(osec.thunks,
-                         [&](std::unique_ptr<RangeExtensionThunk<E>> &thunk) {
-    i64 i = 0;
-    for (i64 j = 0; j < thunk->symbols.size(); j++) {
-      if (thunk->used[j]) {
-        thunk->symbol_map[j] = i;
-        thunk->symbols[i] = thunk->symbols[j];
-        i++;
-      }
-    }
-    thunk->symbols.resize(i);
-  });
-}
-
-static void shrink_section(Context<E> &ctx, OutputSection<E> &osec) {
-  std::span<std::unique_ptr<RangeExtensionThunk<E>>> thunks = osec.thunks;
-  std::span<InputSection<E> *> members = osec.members;
-
-  i64 offset = 0;
-
-  auto add_thunk = [&]() {
-    thunks[0]->offset = offset;
-    offset += thunks[0]->size();
-    thunks = thunks.subspan(1);
-  };
-
-  auto add_isec = [&]() {
-    offset = align_to(offset, members[0]->shdr.sh_addralign);
-    members[0]->offset = offset;
-    offset += members[0]->shdr.sh_size;
-    members = members.subspan(1);
-  };
-
-  while (!thunks.empty() && !members.empty()) {
-    if (thunks[0]->offset < members[0]->offset)
-      add_thunk();
-    else
-      add_isec();
-  }
-
-  while (!thunks.empty())
-    add_thunk();
-  while (!members.empty())
-    add_isec();
-
-  assert(offset <= osec.shdr.sh_size);
-  osec.shdr.sh_size = offset;
-}
-
-// ARM64's call/jump instructions take 27 bits displacement, so they
-// can refer only up to ±128 MiB. If a branch target is further than
-// that, we need to let it branch to a linker-synthesized code
-// sequence that construct a full 32 bit address in a register and
-// jump there. That linker-synthesized code is called "thunk".
-i64 create_range_extension_thunks(Context<E> &ctx) {
-  Timer t(ctx, "create_range_extension_thunks");
-
-  // First, we create thunks with a pessimistic assumption that all
-  // out-of-section relocations would need thunks. To do so, we start
-  // with an initial layout in which output sections are separated far
-  // apart.
-  for (i64 i = 0; Chunk<E> *chunk : ctx.chunks)
-    chunk->shdr.sh_addr = i++ << 31;
-
-  std::vector<OutputSection<E> *> sections;
-  for (std::unique_ptr<OutputSection<E>> &osec : ctx.output_sections)
-    if (!osec->members.empty() && (osec->shdr.sh_flags & SHF_EXECINSTR))
-      sections.push_back(osec.get());
-
-  for (OutputSection<E> *osec : sections)
-    create_thunks(ctx, *osec);
-
-  // Recompute file layout.
-  set_osec_offsets(ctx);
-
-  // Based on the current file layout, remove thunk symbols that turned
-  // to be unnecessary.
-  tbb::parallel_for_each(sections, [&](OutputSection<E> *osec) {
-    mark_thunk_symbols(ctx, *osec);
-  });
-
-  // Recompute output section sizes that contain thunks. New section
-  // sizes must be equal to or smaller than previous values, so all
-  // relocations that were previously reachable will still be reachable
-  // after this step.
-  for (OutputSection<E> *osec : sections)
-    shrink_section(ctx, *osec);
-
-  // Compute the final layout.
-  return set_osec_offsets(ctx);
-}
-
 void RangeExtensionThunk<E>::copy_buf(Context<E> &ctx) {
   u8 *buf = ctx.buf + output_section.shdr.sh_offset + offset;
 
-  static const u8 insn[] = {
-    0x10, 0x00, 0x00, 0x90, // adrp x16, 0   # R_AARCH64_ADR_PREL_PG_HI21
-    0x10, 0x02, 0x00, 0x91, // add  x16, x16 # R_AARCH64_ADD_ABS_LO12_NC
-    0x00, 0x02, 0x1f, 0xd6, // br   x16
+  static const u32 data[] = {
+    0x90000010, // adrp x16, 0   # R_AARCH64_ADR_PREL_PG_HI21
+    0x91000210, // add  x16, x16 # R_AARCH64_ADD_ABS_LO12_NC
+    0xd61f0200, // br   x16
   };
 
-  static_assert(ENTRY_SIZE == sizeof(insn));
+  static_assert(ENTRY_SIZE == sizeof(data));
 
   for (i64 i = 0; i < symbols.size(); i++) {
     u64 S = symbols[i]->get_addr(ctx);
     u64 P = output_section.shdr.sh_addr + offset + i * ENTRY_SIZE;
 
     u8 *loc = buf + i * ENTRY_SIZE;
-    memcpy(loc , insn, sizeof(insn));
-    write_adr(loc, bits(page(S) - page(P), 32, 12));
-    *(u32 *)(loc + 4) |= bits(S, 11, 0) << 10;
+    memcpy(loc , data, sizeof(data));
+    write_adrp(loc, page(S) - page(P));
+    *(ul32 *)(loc + 4) |= bits(S, 11, 0) << 10;
   }
 }
 
-void write_thunks(Context<E> &ctx) {
-  Timer t(ctx, "write_thunks");
-
-  tbb::parallel_for_each(ctx.output_sections,
-                         [&](std::unique_ptr<OutputSection<E>> &osec) {
-    tbb::parallel_for_each(osec->thunks,
-                           [&](std::unique_ptr<RangeExtensionThunk<E>> &thunk) {
-      thunk->copy_buf(ctx);
-    });
-  });
-}
-
 } // namespace mold::elf
