@@ -1,8 +1,11 @@
 #pragma once
 
-#include "byteorder.h"
+#include "inttypes.h"
 
+#include <array>
 #include <atomic>
+#include <bit>
+#include <bitset>
 #include <cassert>
 #include <cstdio>
 #include <cstring>
@@ -10,6 +13,7 @@
 #include <filesystem>
 #include <iostream>
 #include <mutex>
+#include <optional>
 #include <span>
 #include <sstream>
 #include <string>
@@ -22,12 +26,30 @@
 #include <unistd.h>
 #include <vector>
 
+#define XXH_INLINE_ALL 1
+#include <xxhash.h>
+
 #ifdef NDEBUG
 #  define unreachable() __builtin_unreachable()
 #else
 #  define unreachable() assert(0 && "unreachable")
 #endif
 
+inline uint64_t hash_string(std::string_view str) {
+  return XXH3_64bits(str.data(), str.size());
+}
+
+class HashCmp {
+public:
+  static size_t hash(const std::string_view &k) {
+    return hash_string(k);
+  }
+
+  static bool equal(const std::string_view &k1, const std::string_view &k2) {
+    return k1 == k2;
+  }
+};
+
 namespace mold {
 
 using namespace std::literals::string_literals;
@@ -49,11 +71,15 @@ inline char *output_tmpfile;
 inline char *socket_tmpfile;
 inline thread_local bool opt_demangle;
 
+inline u8 *output_buffer_start = nullptr;
+inline u8 *output_buffer_end = nullptr;
+
 extern const std::string mold_version;
 
 std::string_view errno_string();
 void cleanup();
 void install_signal_handler();
+i64 get_default_thread_count();
 
 //
 // Error output
@@ -83,11 +109,18 @@ private:
   std::stringstream ss;
 };
 
+template <typename C>
+static std::string add_color(C &ctx, std::string msg) {
+  if (ctx.arg.color_diagnostics)
+    return "mold: \033[0;1;31m" + msg + ":\033[0m ";
+  return "mold: " + msg + ": ";
+}
+
 template <typename C>
 class Fatal {
 public:
   Fatal(C &ctx) : out(ctx, std::cerr) {
-    out << "mold: ";
+    out << add_color(ctx, "fatal");
   }
 
   [[noreturn]] ~Fatal() {
@@ -109,12 +142,12 @@ template <typename C>
 class Error {
 public:
   Error(C &ctx) : out(ctx, std::cerr) {
-    if (ctx.arg.color_diagnostics)
-      out << "mold: \033[0;1;31merror:\033[0m ";
-    else
-      out << "mold: error: ";
-
-    ctx.has_error = true;
+    if (ctx.arg.noinhibit_exec) {
+      out << add_color(ctx, "warning");
+    } else {
+      out << add_color(ctx, "error");
+      ctx.has_error = true;
+    }
   }
 
   template <class T> Error &operator<<(T &&val) {
@@ -131,16 +164,10 @@ class Warn {
 public:
   Warn(C &ctx) : out(ctx, std::cerr) {
     if (ctx.arg.fatal_warnings) {
-      if (ctx.arg.color_diagnostics)
-        out << "mold: \033[0;1;31merror:\033[0m ";
-      else
-        out << "mold: error: ";
+      out << add_color(ctx, "error");
       ctx.has_error = true;
     } else {
-      if (ctx.arg.color_diagnostics)
-        out << "mold: \033[0;1;35mwarning:\033[0m ";
-      else
-        out << "mold: warning: ";
+      out << add_color(ctx, "warning");
     }
   }
 
@@ -157,36 +184,52 @@ private:
 // Utility functions
 //
 
+// Some C++ libraries haven't implemented std::has_single_bit yet.
+inline bool has_single_bit(u64 val) {
+  return std::popcount(val) == 1;
+}
+
+// Some C++ libraries haven't implemented std::bit_ceil yet.
+inline u64 bit_ceil(u64 val) {
+  if (has_single_bit(val))
+    return val;
+  return (u64)1 << (64 - std::countl_zero(val));
+}
+
 inline u64 align_to(u64 val, u64 align) {
   if (align == 0)
     return val;
-  assert(__builtin_popcount(align) == 1);
+  assert(has_single_bit(align));
   return (val + align - 1) & ~(align - 1);
 }
 
 inline u64 align_down(u64 val, u64 align) {
-  assert(__builtin_popcount(align) == 1);
+  assert(has_single_bit(align));
   return val & ~(align - 1);
 }
 
-inline u64 next_power_of_two(u64 val) {
-  assert(val >> 63 == 0);
-  if (val == 0 || val == 1)
-    return 1;
-  return (u64)1 << (64 - __builtin_clzl(val - 1));
+inline u64 bit(u64 val, i64 pos) {
+  return (val >> pos) & 1;
+};
+
+// Returns [hi:lo] bits of val.
+inline u64 bits(u64 val, u64 hi, u64 lo) {
+  return (val >> lo) & (((u64)1 << (hi - lo + 1)) - 1);
 }
 
+inline i64 sign_extend(u64 val, i64 size) {
+  return (i64)(val << (63 - size)) >> (63 - size);
+};
+
 template <typename T, typename Compare = std::less<T>>
-void update_minimum(std::atomic<T> &atomic, u64 new_val,
-                    Compare cmp = {}) {
+void update_minimum(std::atomic<T> &atomic, u64 new_val, Compare cmp = {}) {
   T old_val = atomic;
   while (cmp(new_val, old_val) &&
          !atomic.compare_exchange_weak(old_val, new_val));
 }
 
 template <typename T, typename Compare = std::less<T>>
-void update_maximum(std::atomic<T> &atomic, u64 new_val,
-                    Compare cmp = {}) {
+void update_maximum(std::atomic<T> &atomic, u64 new_val, Compare cmp = {}) {
   T old_val = atomic;
   while (cmp(old_val, new_val) &&
          !atomic.compare_exchange_weak(old_val, new_val));
@@ -215,14 +258,19 @@ inline void sort(T &vec, U less) {
   std::stable_sort(vec.begin(), vec.end(), less);
 }
 
-inline i64 write_string(u8 *buf, std::string_view str) {
+template <typename T>
+inline void remove_duplicates(std::vector<T> &vec) {
+  vec.erase(std::unique(vec.begin(), vec.end()), vec.end());
+}
+
+inline i64 write_string(void *buf, std::string_view str) {
   memcpy(buf, str.data(), str.size());
-  buf[str.size()] = '\0';
+  *((u8 *)buf + str.size()) = '\0';
   return str.size() + 1;
 }
 
 template <typename T>
-inline i64 write_vector(u8 *buf, const std::vector<T> &vec) {
+inline i64 write_vector(void *buf, const std::vector<T> &vec) {
   i64 sz = vec.size() * sizeof(T);
   memcpy(buf, vec.data(), sz);
   return sz;
@@ -258,13 +306,16 @@ inline u64 read_uleb(u8 *&buf) {
   return val;
 }
 
+inline u64 read_uleb(u8 const*&buf) {
+  return read_uleb(const_cast<u8 *&>(buf));
+}
+
 inline i64 uleb_size(u64 val) {
-  i64 i = 0;
-  do {
-    i++;
-    val >>= 7;
-  } while (val);
-  return i;
+#pragma unroll
+  for (int i = 1; i < 9; i++)
+    if (val < ((u64)1 << (7 * i)))
+      return i;
+  return 9;
 }
 
 template <typename C>
@@ -280,6 +331,11 @@ std::string_view save_string(C &ctx, const std::string &str) {
 // Concurrent Map
 //
 
+// This is an implementation of a fast concurrent hash map. Unlike
+// ordinary hash tables, this impl just aborts if it becomes full.
+// So you need to give a correct estimation of the final size before
+// using it. We use this hash map to uniquify pieces of data in
+// mergeable sections.
 template <typename T>
 class ConcurrentMap {
 public:
@@ -292,7 +348,7 @@ public:
   ~ConcurrentMap() {
     if (keys) {
       free((void *)keys);
-      free((void *)sizes);
+      free((void *)key_sizes);
       free((void *)values);
     }
   }
@@ -300,11 +356,11 @@ public:
   void resize(i64 nbuckets) {
     this->~ConcurrentMap();
 
-    nbuckets = std::max<i64>(MIN_NBUCKETS, next_power_of_two(nbuckets));
+    nbuckets = std::max<i64>(MIN_NBUCKETS, bit_ceil(nbuckets));
 
     this->nbuckets = nbuckets;
     keys = (std::atomic<const char *> *)calloc(nbuckets, sizeof(keys[0]));
-    sizes = (u32 *)calloc(nbuckets, sizeof(sizes[0]));
+    key_sizes = (u32 *)calloc(nbuckets, sizeof(key_sizes[0]));
     values = (T *)calloc(nbuckets, sizeof(values[0]));
   }
 
@@ -312,29 +368,28 @@ public:
     if (!keys)
       return {nullptr, false};
 
-    assert(__builtin_popcount(nbuckets) == 1);
+    assert(has_single_bit(nbuckets));
     i64 idx = hash & (nbuckets - 1);
     i64 retry = 0;
 
     while (retry < MAX_RETRY) {
       const char *ptr = keys[idx];
-      if (ptr == locked) {
-#ifdef __x86_64__
-        asm volatile("pause" ::: "memory");
-#endif
+      if (ptr == marker) {
+        pause();
         continue;
       }
 
       if (ptr == nullptr) {
-        if (!keys[idx].compare_exchange_weak(ptr, locked))
+        if (!keys[idx].compare_exchange_weak(ptr, marker))
           continue;
         new (values + idx) T(val);
-        sizes[idx] = key.size();
+        key_sizes[idx] = key.size();
         keys[idx] = key.data();
         return {values + idx, true};
       }
 
-      if (key.size() == sizes[idx] && memcmp(ptr, key.data(), sizes[idx]) == 0)
+      if (key.size() == key_sizes[idx] &&
+          memcmp(ptr, key.data(), key_sizes[idx]) == 0)
         return {values + idx, false};
 
       u64 mask = nbuckets / NUM_SHARDS - 1;
@@ -347,7 +402,7 @@ public:
   }
 
   bool has_key(i64 idx) {
-    return keys[idx];
+    return keys[idx].load(std::memory_order_relaxed);
   }
 
   static constexpr i64 MIN_NBUCKETS = 2048;
@@ -356,64 +411,45 @@ public:
 
   i64 nbuckets = 0;
   std::atomic<const char *> *keys = nullptr;
-  u32 *sizes = nullptr;
+  u32 *key_sizes = nullptr;
   T *values = nullptr;
 
 private:
-  static constexpr const char *locked = "marker";
+  static void pause() {
+#if defined(__x86_64__)
+    asm volatile("pause");
+#elif defined(__aarch64__)
+    asm volatile("yield");
+#endif
+  }
+
+  static constexpr const char *marker = "marker";
 };
 
 //
-// Bit vector
+// output-file.h
 //
 
-class BitVector {
-  class BitRef {
-  public:
-    BitRef(u8 &byte, u8 bitpos) : byte(byte), bitpos(bitpos) {}
-
-    BitRef &operator=(bool val) {
-      if (val)
-        byte |= (1 << bitpos);
-      else
-        byte &= ~(1 << bitpos);
-      return *this;
-    }
-
-    BitRef &operator=(const BitRef &other) {
-      *this = (bool)other;
-      return *this;
-    }
-
-    operator bool() const {
-      return byte & (1 << bitpos);
-    }
-
-  private:
-    u8 &byte;
-    u8 bitpos;
-  };
-
+template <typename C>
+class OutputFile {
 public:
-  void resize(i64 size) {
-    vec.reset(new u8[(size + 7) / 8]);
-    memset(vec.get(), 0, (size + 7) / 8);
-  }
+  static std::unique_ptr<OutputFile<C>>
+  open(C &ctx, std::string path, i64 filesize, i64 perm);
 
-  BitRef operator[](i64 i) {
-    return BitRef(vec[i / 8], i % 8);
-  }
+  virtual void close(C &ctx) = 0;
+  virtual ~OutputFile() = default;
 
-private:
-  std::unique_ptr<u8[]> vec = nullptr;
+  u8 *buf = nullptr;
+  std::string path;
+  i64 filesize;
+  bool is_mmapped;
+  bool is_unmapped = false;
+
+protected:
+  OutputFile(std::string path, i64 filesize, bool is_mmapped)
+    : path(path), filesize(filesize), is_mmapped(is_mmapped) {}
 };
 
-//
-// threads.cc
-//
-
-void set_thread_count(i64 n);
-
 //
 // hyperloglog.cc
 //
@@ -423,7 +459,7 @@ public:
   HyperLogLog() : buckets(NBUCKETS) {}
 
   void insert(u32 hash) {
-    update_maximum(buckets[hash & (NBUCKETS - 1)], __builtin_clz(hash) + 1);
+    update_maximum(buckets[hash & (NBUCKETS - 1)], std::countl_zero(hash) + 1);
   }
 
   i64 get_cardinality() const;
@@ -440,6 +476,65 @@ private:
   std::vector<std::atomic_uint8_t> buckets;
 };
 
+//
+// glob.cc
+//
+
+class Glob {
+  typedef enum { STRING, STAR, QUESTION, BRACKET } Kind;
+
+  struct Element {
+    Element(Kind k) : kind(k) {}
+    Kind kind;
+    std::string str;
+    std::bitset<256> bitset;
+  };
+
+public:
+  static std::optional<Glob> compile(std::string_view pat);
+  bool match(std::string_view str);
+
+private:
+  Glob(std::vector<Element> &&vec) : elements(vec) {}
+  static bool do_match(std::string_view str, std::span<Element> elements);
+
+  std::vector<Element> elements;
+};
+
+//
+// multi-glob.cc
+//
+
+class MultiGlob {
+public:
+  bool add(std::string_view pat, u32 val);
+  void compile();
+  bool empty() const { return strings.empty(); }
+  std::optional<u32> find(std::string_view str);
+
+private:
+  struct TrieNode {
+    u32 value = -1;
+    TrieNode *suffix_link = nullptr;
+    std::unique_ptr<TrieNode> children[256];
+  };
+
+  void fix_suffix_links(TrieNode &node);
+  void fix_values();
+
+  std::vector<std::string> strings;
+  std::unique_ptr<TrieNode> root;
+  std::vector<std::pair<Glob, u32>> globs;
+  std::vector<u32> values;
+  bool compiled = false;
+};
+
+//
+// uuid.cc
+//
+
+std::array<u8, 16> get_uuid_v4();
+
 //
 // filepath.cc
 //
@@ -465,7 +560,7 @@ std::string_view demangle(std::string_view name);
 
 class ZlibCompressor {
 public:
-  ZlibCompressor(std::string_view input);
+  ZlibCompressor(u8 *buf, i64 size);
   void write_to(u8 *buf);
   i64 size() const;
 
@@ -571,24 +666,24 @@ private:
 
 // TarFile is a class to create a tar file.
 //
-// If you pass `--reproduce=repro.tar` to mold, mold collects all
-// input files and put them into `repro.tar`, so that it is easy to
+// If you pass `--repro` to mold, mold collects all input files and
+// put them into `<output-file-path>.repro.tar`, so that it is easy to
 // run the same command with the same command line arguments.
-class TarFile {
+class TarWriter {
 public:
-  TarFile(std::string basedir) : basedir(basedir) {}
+  static std::unique_ptr<TarWriter>
+  open(std::string output_path, std::string basedir);
+
+  ~TarWriter();
   void append(std::string path, std::string_view data);
-  void write_to(u8 *buf);
-  i64 size() const { return size_; }
 
 private:
   static constexpr i64 BLOCK_SIZE = 512;
 
-  std::string encode_path(std::string path);
+  TarWriter(FILE *out, std::string basedir) : out(out), basedir(basedir) {}
 
+  FILE *out = nullptr;
   std::string basedir;
-  std::vector<std::pair<std::string, std::string_view>> contents;
-  i64 size_ = BLOCK_SIZE * 2;
 };
 
 //
@@ -611,21 +706,39 @@ public:
     return std::string_view((char *)data, size);
   }
 
+  i64 get_offset() const {
+    return parent ? (data - parent->data + parent->get_offset()) : 0;
+  }
+
+  // Returns a string that uniquely identify a file that is possibly
+  // in an archive.
+  std::string get_identifier() const {
+    if (parent) {
+      // We use the file offset within an archive as an identifier
+      // because archive members may have the same name.
+      return parent->name + ":" + std::to_string(get_offset());
+    }
+
+    if (thin_parent) {
+      // If this is a thin archive member, the filename part is
+      // guaranteed to be unique.
+      return thin_parent->name + ":" + name;
+    }
+    return name;
+  }
+
   std::string name;
   u8 *data = nullptr;
   i64 size = 0;
   i64 mtime = 0;
   bool given_fullpath = true;
   MappedFile *parent = nullptr;
+  MappedFile *thin_parent = nullptr;
+  int fd = -1;
 };
 
 template <typename C>
 MappedFile<C> *MappedFile<C>::open(C &ctx, std::string path) {
-  MappedFile *mf = new MappedFile;
-  mf->name = path;
-
-  ctx.mf_pool.push_back(std::unique_ptr<MappedFile>(mf));
-
   if (path.starts_with('/') && !ctx.arg.chroot.empty())
     path = ctx.arg.chroot + "/" + path_clean(path);
 
@@ -637,6 +750,10 @@ MappedFile<C> *MappedFile<C>::open(C &ctx, std::string path) {
   if (fstat(fd, &st) == -1)
     Fatal(ctx) << path << ": fstat failed: " << errno_string();
 
+  MappedFile *mf = new MappedFile;
+  ctx.mf_pool.push_back(std::unique_ptr<MappedFile>(mf));
+
+  mf->name = path;
   mf->size = st.st_size;
 
 #ifdef __APPLE__
@@ -659,7 +776,7 @@ template <typename C>
 MappedFile<C> *MappedFile<C>::must_open(C &ctx, std::string path) {
   if (MappedFile *mf = MappedFile::open(ctx, path))
     return mf;
-  Fatal(ctx) << "cannot open " << path;
+  Fatal(ctx) << "cannot open " << path << ": " << errno_string();
 }
 
 template <typename C>
