@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -40,7 +43,7 @@ rm -f $t/e.a
 ar rcs $t/e.a $t/b.o $t/c.o $t/d.o
 
 $CC -B. -o $t/exe $t/a.o $t/e.a
-$t/exe | grep -q '5 0 0 -1'
+$QEMU $t/exe | grep -q '5 0 0 -1'
 
 cat <<EOF | $CC -fcommon -xc -c -o $t/f.o -
 int bar = 0;
@@ -52,6 +55,6 @@ rm -f $t/f.a
 ar rcs $t/f.a $t/b.o $t/f.o
 
 $CC -B. -o $t/exe $t/a.o $t/f.a
-$t/exe | grep -q '5 0 7 2'
+$QEMU $t/exe | grep -q '5 0 7 2'
 
 echo OK
