@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -28,6 +31,6 @@ EOF
 $CC -B. -o $t/exe $t/a.o -pie -Wl,-defsym=foo=16 \
   -Wl,-defsym=bar=0x2000 -Wl,-defsym=baz=print
 
-$t/exe | grep -q '^Hello 0x10 0x2000$'
+$QEMU $t/exe | grep -q '^Hello 0x10 0x2000$'
 
 echo OK
