@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -18,13 +21,13 @@ int main() {
 EOF
 
 ! $CC -B. -o $t/exe $t/a.o -Wl,-no-demangle 2> $t/log || false
-grep -q 'undefined symbol: .*: _Z3fooii' $t/log
+grep -q 'undefined symbol: _Z3fooii$' $t/log
 
 ! $CC -B. -o $t/exe $t/a.o -Wl,-demangle 2> $t/log || false
-grep -Pq 'undefined symbol: .*: foo\(int, int\)' $t/log
+grep -Eq 'undefined symbol: foo\(int, int\)$' $t/log
 
 ! $CC -B. -o $t/exe $t/a.o 2> $t/log || false
-grep -Pq 'undefined symbol: .*: foo\(int, int\)' $t/log
+grep -Eq 'undefined symbol: foo\(int, int\)$' $t/log
 
 cat <<EOF | $CC -c -o $t/b.o -xc -
 extern int Pi;
@@ -34,6 +37,6 @@ int main() {
 EOF
 
 ! $CC -B. -o $t/exe $t/b.o -Wl,-demangle 2> $t/log || false
-grep -q 'undefined symbol: .*: Pi' $t/log
+grep -q 'undefined symbol: Pi$' $t/log
 
 echo OK
