@@ -1,15 +1,20 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
+[ $MACHINE = riscv64 ] && { echo skipped; exit; }
+
 cat <<EOF | $CC -o $t/a.o -c -x assembler -Wa,-L -
   .text
   .globl _start
@@ -21,25 +26,25 @@ foo:
   nop
 EOF
 
-"$mold" -o $t/exe $t/a.o
+./mold -o $t/exe $t/a.o
 readelf --symbols $t/exe > $t/log
 fgrep -q _start $t/log
 fgrep -q foo $t/log
 fgrep -q .Lbar $t/log
 
-"$mold" -o $t/exe $t/a.o --discard-locals
+./mold -o $t/exe $t/a.o --discard-locals
 readelf --symbols $t/exe > $t/log
 fgrep -q _start $t/log
 fgrep -q foo $t/log
 ! fgrep -q .Lbar $t/log || false
 
-"$mold" -o $t/exe $t/a.o --discard-all
+./mold -o $t/exe $t/a.o --discard-all
 readelf --symbols $t/exe > $t/log
 fgrep -q _start $t/log
 ! fgrep -q foo $t/log || false
 ! fgrep -q .Lbar $t/log || false
 
-"$mold" -o $t/exe $t/a.o --strip-all
+./mold -o $t/exe $t/a.o --strip-all
 readelf --symbols $t/exe > $t/log
 ! fgrep -q _start $t/log || false
 ! fgrep -q foo $t/log || false
