@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -17,7 +20,7 @@ main:
   nop
 EOF
 
-! "$mold" -o $t/exe $t/a.o $t/a.o 2> $t/log || false
+! ./mold -o $t/exe $t/a.o $t/a.o 2> $t/log || false
 grep -q 'duplicate symbol: .*\.o: .*\.o: main' $t/log
 
 echo OK
