@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -25,10 +28,10 @@ _start:
 EOF
 
 $CC -shared -fPIC -o $t/b.so -xc /dev/null
-"$mold" -o $t/exe $t/a.o $t/b.so --export-dynamic
+./mold -o $t/exe $t/a.o $t/b.so --export-dynamic
 
 readelf --dyn-syms $t/exe > $t/log
-fgrep -q 'NOTYPE  GLOBAL DEFAULT    6 bar' $t/log
-fgrep -q 'NOTYPE  GLOBAL DEFAULT    6 _start' $t/log
+grep -Pq 'NOTYPE  GLOBAL DEFAULT    \d+ bar' $t/log
+grep -Pq 'NOTYPE  GLOBAL DEFAULT    \d+ _start' $t/log
 
 echo OK
