@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -42,7 +45,7 @@ grep -qv live_var1 $t/log.1
 grep -qv live_var2 $t/log.1
 grep -qv dead_var1 $t/log.1
 grep -qv dead_var2 $t/log.1
-$t/exe1 | grep -q '1 2'
+$QEMU $t/exe1 | grep -q '1 2'
 
 $CXX -B. -o $t/exe2 $t/a.cc -ffunction-sections -fdata-sections -Wl,-gc-sections
 
@@ -55,6 +58,6 @@ grep -q  live_var1 $t/log.2
 grep -q  live_var2 $t/log.2
 grep -qv dead_var1 $t/log.2
 grep -qv dead_var2 $t/log.2
-$t/exe2 | grep -q '1 2'
+$QEMU $t/exe2 | grep -q '1 2'
 
 echo OK
