@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -22,6 +25,7 @@ int main() {
 EOF
 
 $CC -B. -o $t/b.so -shared $t/a.o
-readelf -W --sections $t/b.so | fgrep -A1 .rela.dyn | fgrep -q .rela.plt
+readelf -W --sections $t/b.so | grep -P -A1 '\.rela?\.dyn' | \
+  grep -Pq '\.rela?\.plt'
 
 echo OK
