@@ -1,18 +1,24 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
+# IFUNC is not supported on RISC-V yet
+[ $MACHINE = riscv64 ] && { echo skipped; exit; }
+
 # Skip if libc is musl because musl does not support GNU FUNC
 echo 'int main() {}' | $CC -o $t/exe -xc -
-ldd $t/exe | grep -q ld-musl && { echo OK; exit; }
+readelf --dynamic $t/exe | grep -q ld-musl && { echo OK; exit; }
 
 cat <<EOF | $CC -c -fPIC -o $t/a.o -xc -
 #include <stdio.h>
@@ -32,6 +38,6 @@ Func *resolve_foobar(void) {
 EOF
 
 $CC -B. -shared -o $t/b.so $t/a.o
-readelf --dyn-syms $t/b.so | grep -Pq '(IFUNC|<OS specific>: 10)\s+GLOBAL DEFAULT   \d+ foobar'
+readelf --dyn-syms $t/b.so | grep -Eq '(IFUNC|<OS specific>: 10)\s+GLOBAL DEFAULT   .* foobar'
 
 echo OK
