@@ -1,15 +1,22 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
+# musl does not support GNU-style init/fini priorities
+echo 'int main() {}' | $CC -o $t/exe -xc -
+readelf --dynamic $t/exe | grep -q ld-musl && { echo skipped; exit; }
+
 cat <<'EOF' | $CC -c -o $t/a.o -xc -
 #include <stdio.h>
 __attribute__((constructor(10000))) void init4() { printf("1"); }
@@ -54,8 +61,7 @@ cat <<EOF | $CC -c -o $t/i.o -xc -
 int main() {}
 EOF
 
-$CC -B. -o $t/exe $t/a.o $t/b.o $t/c.o $t/d.o \
-  $t/e.o $t/f.o $t/g.o $t/h.o $t/i.o
-$t/exe | grep -q '21348756'
+$CC -B. -o $t/exe $t/a.o $t/b.o $t/c.o $t/d.o $t/e.o $t/f.o $t/g.o $t/h.o $t/i.o
+$QEMU $t/exe | grep -q '21348756'
 
 echo OK
