@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -19,6 +22,6 @@ void foo() {
 EOF
 
 $CC -B. -shared -o $t/b.so $t/a.o -Wl,-z,interpose
-readelf --dynamic $t/b.so | grep -q 'Flags: INTERPOSE'
+readelf --dynamic $t/b.so | grep -q 'Flags:.*INTERPOSE'
 
 echo OK
