@@ -1,15 +1,20 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
+[ $MACHINE = x86_64 ] || { echo skipped; exit; }
+
 cat <<EOF | $CC -o $t/a.o -c -x assembler -
 .text
 .globl _start
@@ -33,12 +38,12 @@ _start:
 .quad 42
 EOF
 
-"$mold" -static -o $t/exe $t/a.o
+./mold -static -o $t/exe $t/a.o
 readelf -W --sections $t/exe > $t/log
 
-grep -Pq '.note.bar\s+NOTE.+000008 00   A  0   0  4' $t/log
-grep -Pq '.note.baz\s+NOTE.+000008 00   A  0   0  8' $t/log
-grep -Pq '.note.nonalloc\s+NOTE.+000008 00      0   0  1' $t/log
+grep -Eq '.note.bar\s+NOTE.+000008 00   A  0   0  4' $t/log
+grep -Eq '.note.baz\s+NOTE.+000008 00   A  0   0  8' $t/log
+grep -Eq '.note.nonalloc\s+NOTE.+000008 00      0   0  1' $t/log
 
 readelf --segments $t/exe > $t/log
 fgrep -q '01     .note.bar' $t/log
