@@ -1,16 +1,19 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
-[ "$(uname -m)" = x86_64 ] || { echo skipped; exit; }
+[ $MACHINE = x86_64 ] || { echo skipped; exit; }
 
 cat <<EOF | $CC -fPIC -shared -o $t/a.so -x assembler -
 .globl ext1, ext2
@@ -29,10 +32,10 @@ _start:
   ret
 EOF
 
-"$mold" --pie -o $t/exe $t/b.o $t/a.so
+./mold --pie -o $t/exe $t/b.o $t/a.so
 
-objdump -d -j .plt.got $t/exe > $t/log
+$OBJDUMP -d -j .plt.got $t/exe > $t/log
 
-grep -Pq '1020:.*jmp.*# 2000 <ext2>' $t/log
+grep -Eq '1020:.*jmp.* <ext2>' $t/log
 
 echo OK
