@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -14,7 +17,7 @@ cat <<EOF | $CC -c -o $t/a.o -xc -
 static void foo() {}
 void bar() {}
 void baz() {}
-int main() {}
+int main() { foo(); }
 EOF
 
 cat <<EOF > $t/symbols
@@ -22,8 +25,7 @@ foo
 baz
 EOF
 
-$CC -B. -o $t/exe $t/a.o \
-  -Wl,--retain-symbols-file=$t/symbols
+$CC -B. -o $t/exe $t/a.o -Wl,--retain-symbols-file=$t/symbols
 readelf --symbols $t/exe > $t/log
 
 ! grep -qw foo $t/log || false
