@@ -1,15 +1,20 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
+[ $MACHINE = x86_64 ] || { echo skipped; exit; }
+
 cat <<'EOF' | $CC -c -o $t/a.o -xc -
 #include <stdint.h>
 #include <stdio.h>
@@ -42,6 +47,6 @@ int main() {
 EOF
 
 $CC -B. -o $t/exe $t/a.o
-$t/exe | grep -q '^0 0 0$'
+$QEMU $t/exe | grep -q '^0 0 0$'
 
 echo OK
