@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -20,8 +23,8 @@ $CC -B. -shared -o $t/b.so $t/a.o
 
 readelf --dyn-syms $t/b.so > $t/log
 
-grep -q '0000000000000000     0 NOTYPE  GLOBAL DEFAULT  UND fn2' $t/log
-grep -Pq 'FUNC    GLOBAL DEFAULT   \d+ fn1' $t/log
+grep -q '00000000     0 NOTYPE  GLOBAL DEFAULT  UND fn2' $t/log
+grep -Eq 'FUNC    GLOBAL DEFAULT .* fn1' $t/log
 
 cat <<EOF | $CC -fPIC -c -o $t/c.o -xc -
 #include <stdio.h>
@@ -39,7 +42,7 @@ int main() {
 EOF
 
 $CC -B. -o $t/exe $t/c.o $t/b.so
-$t/exe | grep -q hello
+$QEMU $t/exe | grep -q hello
 ! readelf --symbols $t/exe | grep -q fn3 || false
 
 echo OK
