@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -38,6 +41,6 @@ fgrep -q 'static-archive/d.a(long-long-long-filename.o)' $t/log
 fgrep -q 'static-archive/d.a(b.o)' $t/log
 fgrep -q static-archive/c.o $t/log
 
-$t/exe | grep -q '8'
+$QEMU $t/exe | grep -q '8'
 
 echo OK
