@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -18,18 +21,24 @@ bar:
 .L.baz:
 EOF
 
-"$mold" -o $t/exe $t/a.o
+./mold -o $t/exe $t/a.o
 readelf --symbols $t/exe > $t/log
 fgrep -q _start $t/log
 fgrep -q foo $t/log
 fgrep -q bar $t/log
-fgrep -q .L.baz $t/log
 
-"$mold" -o $t/exe $t/a.o -strip-all
+if [ $MACHINE '!=' riscv64 ]; then
+  fgrep -q .L.baz $t/log
+fi
+
+./mold -o $t/exe $t/a.o -strip-all
 readelf --symbols $t/exe > $t/log
 ! fgrep -q _start $t/log || false
 ! fgrep -q foo $t/log || false
 ! fgrep -q bar $t/log || false
-! fgrep -q .L.baz $t/log || false
+
+if [ $MACHINE '!=' riscv64 ]; then
+  ! fgrep -q .L.baz $t/log || false
+fi
 
 echo OK
