@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -48,6 +51,6 @@ rm -f $t/x.a
 ar rcs $t/x.a $t/a.o $t/b.o $t/c.o
 
 $CC -B. -o $t/exe $t/d.o $t/x.a
-$t/exe | grep -q '^0 0 0 5$'
+$QEMU $t/exe | grep -q '^0 0 0 5$'
 
 echo OK
