@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -39,10 +42,10 @@ rm -f $t/d.a
 
 $CC -B. -Wl,--trace -o $t/exe $t/d.o $t/d.a > $t/log
 
-grep -Pq 'thin-archive/d.a\(.*long-long-long-filename.o\)' $t/log
-grep -Pq 'thin-archive/d.a\(.*/b.o\)' $t/log
+grep -Eq 'thin-archive/d.a\(.*long-long-long-filename.o\)' $t/log
+grep -Eq 'thin-archive/d.a\(.*/b.o\)' $t/log
 fgrep -q thin-archive/d.o $t/log
 
-$t/exe | grep -q 15
+$QEMU $t/exe | grep -q 15
 
 echo OK
