@@ -1,25 +1,28 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
-if [ "$(uname -m)" = x86_64 ]; then
+if [ $MACHINE = x86_64 ]; then
   dialect=gnu
-elif [ "$(uname -m)" = aarch64 ]; then
+elif [ $MACHINE = aarch64 ]; then
   dialect=trad
 else
   echo skipped
-  exit 0
+  exit
 fi
 
-cat <<EOF | gcc -ftls-model=initial-exec -mtls-dialect=$dialect -fPIC -c -o $t/a.o -xc -
+cat <<EOF | $GCC -ftls-model=initial-exec -mtls-dialect=$dialect -fPIC -c -o $t/a.o -xc -
 #include <stdio.h>
 
 static _Thread_local int foo;
@@ -37,7 +40,7 @@ EOF
 
 $CC -B. -shared -o $t/b.so $t/a.o
 
-cat <<EOF | gcc -c -o $t/c.o -xc -
+cat <<EOF | $GCC -c -o $t/c.o -xc -
 #include <stdio.h>
 
 _Thread_local int baz;
@@ -55,9 +58,9 @@ int main() {
 EOF
 
 $CC -B. -o $t/exe $t/b.so $t/c.o
-$t/exe | grep -q '^0 0 3 5 7$'
+$QEMU $t/exe | grep -q '^0 0 3 5 7$'
 
 $CC -B. -o $t/exe $t/b.so $t/c.o -Wl,-no-relax
-$t/exe | grep -q '^0 0 3 5 7$'
+$QEMU $t/exe | grep -q '^0 0 3 5 7$'
 
 echo OK
