@@ -1,25 +1,28 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
-if [ "$(uname -m)" = x86_64 ]; then
+if [ $MACHINE = x86_64 ]; then
   dialect=gnu
-elif [ "$(uname -m)" = aarch64 ]; then
+elif [ $MACHINE = aarch64 ]; then
   dialect=trad
 else
   echo skipped
-  exit 0
+  exit
 fi
 
-cat <<EOF | gcc -ftls-model=local-exec -mtls-dialect=$dialect -fPIC -c -o $t/a.o -xc -
+cat <<EOF | $GCC -ftls-model=local-exec -mtls-dialect=$dialect -fPIC -c -o $t/a.o -xc -
 #include <stdio.h>
 
 extern _Thread_local int foo;
@@ -36,14 +39,14 @@ int main() {
 }
 EOF
 
-cat <<EOF | gcc -ftls-model=local-exec -mtls-dialect=$dialect -fPIC -c -o $t/b.o -xc -
+cat <<EOF | $GCC -ftls-model=local-exec -mtls-dialect=$dialect -fPIC -c -o $t/b.o -xc -
 _Thread_local int foo = 3;
 EOF
 
 $CC -B. -o $t/exe $t/a.o $t/b.o
-$t/exe | grep -q '3 5 3 5'
+$QEMU $t/exe | grep -q '3 5 3 5'
 
 $CC -B. -o $t/exe $t/a.o $t/b.o -Wl,-no-relax
-$t/exe | grep -q '3 5 3 5'
+$QEMU $t/exe | grep -q '3 5 3 5'
 
 echo OK
