@@ -1,25 +1,28 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
-if [ "$(uname -m)" = x86_64 ]; then
+if [ $MACHINE = x86_64 ]; then
   dialect=gnu2
-elif [ "$(uname -m)" = aarch64 ]; then
+elif [ $MACHINE = aarch64 ]; then
   dialect=desc
 else
   echo skipped
-  exit 0
+  exit
 fi
 
-cat <<EOF | gcc -fPIC -mtls-dialect=$dialect -c -o $t/a.o -xc -
+cat <<EOF | $GCC -fPIC -mtls-dialect=$dialect -c -o $t/a.o -xc -
 extern _Thread_local int foo;
 
 int get_foo() {
@@ -33,7 +36,7 @@ int get_bar() {
 }
 EOF
 
-cat <<EOF | gcc -fPIC -mtls-dialect=$dialect -c -o $t/b.o -xc -
+cat <<EOF | $GCC -fPIC -mtls-dialect=$dialect -c -o $t/b.o -xc -
 #include <stdio.h>
 
 _Thread_local int foo;
@@ -49,17 +52,17 @@ int main() {
 EOF
 
 $CC -B. -o $t/exe $t/a.o $t/b.o
-$t/exe | grep -q '42 5'
+$QEMU $t/exe | grep -q '42 5'
 
 $CC -B. -o $t/exe $t/a.o $t/b.o -Wl,-no-relax
-$t/exe | grep -q '42 5'
+$QEMU $t/exe | grep -q '42 5'
 
 $CC -B. -shared -o $t/c.so $t/a.o
 $CC -B. -o $t/exe $t/b.o $t/c.so
-$t/exe | grep -q '42 5'
+$QEMU $t/exe | grep -q '42 5'
 
 $CC -B. -shared -o $t/c.so $t/a.o -Wl,-no-relax
 $CC -B. -o $t/exe $t/b.o $t/c.so -Wl,-no-relax
-$t/exe | grep -q '42 5'
+$QEMU $t/exe | grep -q '42 5'
 
 echo OK
