@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -28,17 +31,17 @@ EOF
 rm -f $t/d.a
 ar cr $t/d.a $t/b.o $t/c.o
 
-"$mold" -static -o $t/exe $t/a.o $t/d.a
+./mold -static -o $t/exe $t/a.o $t/d.a
 readelf --symbols $t/exe > $t/log
 ! grep -q foo $t/log || false
 ! grep -q bar $t/log || false
 
-"$mold" -static -o $t/exe $t/a.o $t/d.a -u foo
+./mold -static -o $t/exe $t/a.o $t/d.a -u foo
 readelf --symbols $t/exe > $t/log
 grep -q foo $t/log
 ! grep -q bar $t/log || false
 
-"$mold" -static -o $t/exe $t/a.o $t/d.a -u foo --undefined=bar
+./mold -static -o $t/exe $t/a.o $t/d.a -u foo --undefined=bar
 readelf --symbols $t/exe > $t/log
 grep -q foo $t/log
 grep -q bar $t/log
