@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
@@ -43,12 +46,12 @@ int main() {
 EOF
 
 $CC -B. -o $t/exe $t/a.o $t/b.o
-$t/exe | grep -q '^foo$'
+$QEMU $t/exe | grep -q '^foo$'
 
 $CC -B. -o $t/exe $t/a.o $t/b.o -Wl,-wrap,foo
-$t/exe | grep -q '^wrap_foo$'
+$QEMU $t/exe | grep -q '^wrap_foo$'
 
 $CC -B. -o $t/exe $t/a.o $t/c.o -Wl,-wrap,foo
-$t/exe | grep -q '^foo$'
+$QEMU $t/exe | grep -q '^foo$'
 
 echo OK
