@@ -1,21 +1,24 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/mold"
 t=out/test/elf/$testname
 mkdir -p $t
 
 # Skip if libc is musl
 echo 'int main() {}' | $CC -o $t/exe -xc -
-ldd $t/exe | grep -q ld-musl && { echo OK; exit; }
+readelf --dynamic $t/exe | grep -q ld-musl && { echo OK; exit; }
 
 # Skip if target is not x86-64
-[ "$(uname -m)" = x86_64 ] || { echo skipped; exit; }
+[ $MACHINE = x86_64 ] || { echo skipped; exit; }
 
 cat <<'EOF' | $CC -c -o $t/a.o -x assembler -
 .globl fn1
@@ -44,7 +47,7 @@ int main() {
 EOF
 
 $CC -B. -pie -o $t/exe $t/a.o $t/b.o
-$t/exe | grep -q 3
+$QEMU $t/exe | grep -q 3
 
 readelf --dynamic $t/exe | fgrep -q '(TEXTREL)'
 readelf --dynamic $t/exe | grep -q '\(FLAGS\).*TEXTREL'
