@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/ld64.mold"
 t=out/test/macho/$testname
 mkdir -p $t
 
@@ -18,7 +21,7 @@ int main() {
 }
 EOF
 
-! clang -fuse-ld="$mold" -o $t/exe $t/a.o 2> $t/log || false
+! clang --ld-path=./ld64 -o $t/exe $t/a.o 2> $t/log || false
 grep -q 'undefined symbol: .*\.o: _foo' $t/log
 
 echo OK
