@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/ld64.mold"
 t=out/test/macho/$testname
 mkdir -p $t
 
@@ -19,12 +22,12 @@ int main() {
 }
 EOF
 
-clang -fuse-ld="$mold" -o $t/exe $t/a.o
+clang --ld-path=./ld64 -o $t/exe $t/a.o
 
 otool -l $t/exe | grep -A5 'segname __PAGEZERO' | \
   grep -q 'vmsize 0x0000000100000000'
 
-clang -fuse-ld="$mold" -o $t/exe $t/a.o -Wl,-pagezero_size,0x10000
+clang --ld-path=./ld64 -o $t/exe $t/a.o -Wl,-pagezero_size,0x10000
 $t/exe | grep -q 'Hello world'
 
 otool -l $t/exe | grep -A5 'segname __PAGEZERO' | \
