@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/ld64.mold"
 t=out/test/macho/$testname
 mkdir -p $t
 
@@ -17,7 +20,7 @@ void hello() {
 }
 EOF
 
-clang -fuse-ld="$mold" -shared -o $t/b.dylib $t/a.o
+clang --ld-path=./ld64 -shared -o $t/b.dylib $t/a.o
 otool -l $t/b.dylib > $t/log
 ! grep -q 'segname: __PAGEZERO' $t/log || false
 
