@@ -1,12 +1,15 @@
 #!/bin/bash
-export LANG=
+export LC_ALL=C
 set -e
-CC="${CC:-cc}"
-CXX="${CXX:-c++}"
-testname=$(basename -s .sh "$0")
+CC="${TEST_CC:-cc}"
+CXX="${TEST_CXX:-c++}"
+GCC="${TEST_GCC:-gcc}"
+GXX="${TEST_GXX:-g++}"
+OBJDUMP="${OBJDUMP:-objdump}"
+MACHINE="${MACHINE:-$(uname -m)}"
+testname=$(basename "$0" .sh)
 echo -n "Testing $testname ... "
 cd "$(dirname "$0")"/../..
-mold="$(pwd)/ld64.mold"
 t=out/test/macho/$testname
 mkdir -p $t
 
@@ -36,10 +39,10 @@ mkdir -p $t/x $t/y
 ar rcs $t/x/libfoo.a $t/a.o
 $CC -shared -o $t/y/libfoo.dylib $t/b.o
 
-clang -fuse-ld="$mold" -o $t/exe $t/c.o -Wl,-L$t/x -Wl,-L$t/y -lfoo
+clang --ld-path=./ld64 -o $t/exe $t/c.o -Wl,-L$t/x -Wl,-L$t/y -lfoo
 $t/exe | grep -q Hello
 
-clang -fuse-ld="$mold" -o $t/exe $t/c.o -Wl,-L$t/x -Wl,-L$t/y -lfoo \
+clang --ld-path=./ld64 -o $t/exe $t/c.o -Wl,-L$t/x -Wl,-L$t/y -lfoo \
  -Wl,-search_paths_first
 $t/exe | grep -q Hello
 
